// File: Main.pas
//
// Description: A sample dialog demonstrating virtual printer integration
// methods and print job information retrieval

unit Main;

interface

uses Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls,
  ShellApi, IniFiles, Registry, Clipbrd;

const
  PrinterName = 'Your Virtual Printer Example';
  RegistryKey = 'Your Virtual Printer Example';
  WindowTitle = 'Sample Client Application';

  FileTypes: array[0..7] of string = (
    'EMF', 'BMP', 'PNG', 'JPEG', 'TIFF', 'PDF', 'SWF', 'TXT');

type
  // Print job info transfer modes
  TTransferModes = (tmThroughCommandLine, tmThroughCopydata, tmThroughClipboard);

  // Main form class
  TMainForm = class(TForm)
    Label1: TLabel;
    ListBox: TListBox;
    Label2: TLabel;
    Bevel: TBevel;
    PrintPrefsButton: TButton;
    FindMoreButton: TButton;
    CloseButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure PrintPrefsButtonClick(Sender: TObject);
    procedure FindMoreButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
  private
    FExePath: string;
    FTransferMode: TTransferModes;
    FMessageID: Integer;
    FInputIniFilePath: string;
    FClipboardFormat: string;
    procedure WMCopyData(var Message: TMessage); message WM_COPYDATA;
    procedure FillInfo;
  public
    procedure WndProc(var Message: TMessage); override;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
var
  Reg: TRegistry;
begin
  // Initialize fields
  FExePath := ExtractFilePath(ParamStr(0));
  FTransferMode := tmThroughCommandLine;
  FMessageID := -1;

  // Set unique window title
  Caption := WindowTitle + ' for ' + PrinterName;

  // Get transfer mode related settings from the printer configuration
  Reg := TRegistry.Create;
  Reg.RootKey := HKEY_LOCAL_MACHINE;
  Reg.Access := KEY_READ;
  if Reg.OpenKey('Software\' + RegistryKey, false) then
  try
    FTransferMode := TTransferModes(Reg.ReadInteger('Transfer Mode'));
    FMessageID := Reg.ReadInteger('Message ID');
    FClipboardFormat := Reg.ReadString('Clipboard Format');
  except
    MessageBox(Handle, 'Failed to read printer configuration',
      WindowTitle, MB_ICONERROR or MB_OK);
  end;
  Reg.Free;

  // If transfer mode is "Command Line" then get print
  // job info file path from the command line
  if FTransferMode = tmThroughCommandLine then
  begin
    FInputIniFilePath := ParamStr(1);
    if FileExists(FInputIniFilePath) then
      FillInfo
    else
      ListBox.Items.Add('Invalid command line parameter!');
  end else
    ListBox.Items.Add('Waiting for a print job...');
end;

procedure TMainForm.WMCopyData(var Message: TMessage);
var
  Data: PCopyDataStruct;
begin
  Data := PCopyDataStruct(Message.LParam);

  // Check if the message has been received from the EMF printer
  if Assigned(Data) and (Data^.dwData = Cardinal(FMessageID)) then
  begin
    // Get the file name from the message data
    FInputIniFilePath := PWideChar(Data^.lpData);

    // Check .ini file existance and parse its content
    if FileExists(FInputIniFilePath) then
    begin
      FillInfo;
      SetForegroundWindow(Handle);
    end else
      MessageBox(Handle, PChar('Ini file ' + FInputIniFilePath +
        ' does not exist'), WindowTitle, MB_ICONERROR or MB_OK);

    // Message is handled
    Message.Result := 1;
  end else
    // Call default handler
    inherited;
end;

procedure TMainForm.WndProc(var Message: TMessage);
var
  CFormat, CHandle: Cardinal;
  ClipBoard: TClipboard;
  s: string;
begin
  // Analyze all messages from WM_APP range, check if the
  // message has been received from the EMF printer
  if (Message.Msg >= $8000) and (Message.Msg <= $BFFF) and
    (Message.Msg = Cardinal(FMessageID)) then
  begin
    // Check FClipboardFormat field
    if Length(FClipboardFormat) = 0 then
      MessageBox(Handle, 'Configuration error: clipboard format is undefined',
        WindowTitle, MB_ICONERROR or MB_OK)
    else begin
      // Register clipboard format
      CFormat := RegisterClipboardFormat(PChar(FClipboardFormat));

      // Read data from the clipboard
      CHandle := INVALID_HANDLE_VALUE;
      ClipBoard := TClipboard.Create;
      try
        try
          Clipboard.Open;
          CHandle := Clipboard.GetAsHandle(CFormat);
          s := PChar(GlobalLock(CHandle));
        finally
          GlobalUnlock(CHandle);
          Clipboard.Close;
          Clipboard.Free;
        end;
      except
        MessageBox(Handle, 'Failed to get clipboard data', WindowTitle,
          MB_ICONERROR or MB_OK);
      end;

      // Check .ini file existance and parse its content
      if Length(s) > 0 then
        if FileExists(s) then
        begin
          FInputIniFilePath := s;
          FillInfo;
          SetForegroundWindow(Handle);
        end else
          MessageBox(Handle, PChar('Ini file ' + s + ' does not exist'),
            WindowTitle, MB_ICONERROR or MB_OK);
    end;

    // Message is handled
    Message.Result := 1;
  end else
    // Call default handler
    inherited;
end;

procedure TMainForm.FillInfo;
var
  s: string;
  Ini: TIniFile;
  Orientation, Collate, Count, i, j: Integer;
begin
  // Clear the ListBox
  ListBox.Items.Clear;

  // Show the transfer mode
  case FTransferMode of
    tmThroughCommandLine: s := 'Through the command line';
    tmThroughCopydata: s := 'Through the WM_COPYDATA message';
    tmThroughClipboard: s := 'Through the clipboard';
  else
    s := 'Unknown';
  end;
  ListBox.Items.Add('Transfer mode = ' + s);
  ListBox.Items.Add('');

  // Show print job info file
  ListBox.Items.Add(FInputIniFilePath);
  ListBox.Items.Add('');

  // Open .ini file
  Ini := TIniFile.Create(FInputIniFilePath);

  // Show device name
  ListBox.Items.Add('Device Name = ' + Ini.ReadString('Device', 'DeviceName', ''));

  // Show print job (document) name
  ListBox.Items.Add('Document Name = ' + Ini.ReadString('Document', 'Name', ''));

  // Show document resolution (dpi)
  ListBox.Items.Add(Format('Horizontal Resolution = %sdpi',
    [Ini.ReadString('Document', 'HorizontalResolution', '')]));
  ListBox.Items.Add(Format('Vertical Resolution = %sdpi',
    [Ini.ReadString('Document', 'VerticalResolution', '')]));

  // Show page orientation (1 - portrait, 2 - landscape)
  Orientation := Ini.ReadInteger('Document', 'Orientation', 0);
  if Orientation = 1 then s := 'Portrait' else
    if Orientation = 2 then s := 'Landscape' else
      s := 'Unknown';
  ListBox.Items.Add('Orientation = ' + s);

  // Show paper form name and size
  ListBox.Items.Add(Format('Paper size = %s (%d x %d mm)', [Ini.ReadString('Document',
    'PaperSizeName', ''), Ini.ReadInteger('Document', 'PaperWidth', 0) div 10,
    Ini.ReadInteger('Document', 'PaperLength', 0) div 10]));

  // Show printer bin
  ListBox.Items.Add('Paper bin = ' + Ini.ReadString('Document', 'BinName', ''));

  // Show print job (document) page count
  ListBox.Items.Add('Document pages = ' + Ini.ReadString('Document', 'Pages', ''));

  // Show print job (document) copies count
  ListBox.Items.Add('Copies = ' + Ini.ReadString('Document', 'Copies', ''));

  // Show print job (document) collate option
  Collate := Ini.ReadInteger('Document', 'Collate', 0);
  if Collate = 0 then ListBox.Items.Add('Collate = No') else
    ListBox.Items.Add('Collate = Yes');

  // Show generated files
  for i := 0 to High(FileTypes) do
  begin
    Count := Ini.ReadInteger(FileTypes[i], 'Count', 0);
    if Count > 0 then
    begin
      ListBox.Items.Add('');
      ListBox.Items.Add(Format('%s files (%d):', [FileTypes[i], Count]));
      for j := 0 to Count - 1 do
        ListBox.Items.Add(Ini.ReadString(FileTypes[i], 'File' + IntToStr(j), ''));
    end;
  end;

  Ini.Free;
end;

procedure TMainForm.ListBoxClick(Sender: TObject);
begin
  // If the clicked item of the listbox contains path of file the
  // file will be opened with default associated application
  with ListBox do
    if FileExists(Items[ItemIndex]) then
      ShellExecute(Handle, 'open', PChar(Items[ItemIndex]), nil, nil, SW_SHOWNORMAL);
end;

procedure TMainForm.PrintPrefsButtonClick(Sender: TObject);
begin
  // Execute properties.exe from the same folder
  if ShellExecute(Handle, 'open', PChar(FExePath + 'properties.exe'),
    nil, PChar(FExePath), SW_SHOWNORMAL) <= 32 then
    MessageBox(Handle, PChar('File properties.exe not found in ' + FExePath),
      WindowTitle, MB_ICONERROR or MB_OK);
end;

procedure TMainForm.FindMoreButtonClick(Sender: TObject);
begin
  // Open virtual printer home page at the web-site
  ShellExecute(Handle, 'open', 'http://www.colorpilot.com/emfprinterpilot.html',
    nil, nil, SW_SHOWNORMAL);
end;

procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

end.
